@@ -36,14 +36,15 @@ use pact_models::provider_states::*;
 use pact_models::v4::interaction::V4Interaction;
 
 use crate::callback_executors::{ProviderStateError, ProviderStateExecutor};
-use crate::messages::{display_message_result, verify_message_from_provider, verify_sync_message_from_provider};
+use crate::messages::{process_message_result, verify_message_from_provider, verify_sync_message_from_provider};
 use crate::pact_broker::{Link, PactVerificationContext, publish_verification_results, TestResult};
 pub use crate::pact_broker::{ConsumerVersionSelector, PactsForVerificationRequest};
 use crate::provider_client::make_provider_request;
-use crate::request_response::display_request_response_result;
+use crate::request_response::process_request_response_result;
 use pact_plugin_driver::plugin_models::{PluginDependency, PluginDependencyType};
 use pact_matching::metrics::{MetricEvent, send_metrics};
 use crate::metrics::VerificationMetrics;
+use crate::verification_result::VerificationExecutionResult;
 
 mod provider_client;
 pub mod pact_broker;
@@ -52,6 +53,7 @@ mod request_response;
 mod messages;
 pub mod selectors;
 pub mod metrics;
+pub mod verification_result;
 
 /// Source for loading pacts
 #[derive(Debug, Clone)]
@@ -383,23 +385,24 @@ async fn verify_interaction<'a, F: RequestFilterExecutor, S: ProviderStateExecut
   result
 }
 
-fn display_result(
+fn generate_display_for_result(
   status: u16,
   status_result: ANSIGenericString<str>,
   header_results: Option<Vec<(String, String, ANSIGenericString<str>)>>,
-  body_result: ANSIGenericString<str>
+  body_result: ANSIGenericString<str>,
+  output: &mut Vec<String>
 ) {
-  println!("    returns a response which");
-  println!("      has status code {} ({})", Style::new().bold().paint(format!("{}", status)),
-      status_result);
+  output.push("    returns a response which".to_string());
+  output.push(format!("      has status code {} ({})", Style::new().bold().paint(format!("{}", status)),
+      status_result));
   if let Some(header_results) = header_results {
-    println!("      includes headers");
+    output.push("      includes headers".to_string());
     for (key, value, result) in header_results {
-      println!("        \"{}\" with value \"{}\" ({})", Style::new().bold().paint(key),
-               Style::new().bold().paint(value), result);
+      output.push(format!("        \"{}\" with value \"{}\" ({})", Style::new().bold().paint(key),
+               Style::new().bold().paint(value), result));
     }
   }
-  println!("      has a matching body ({})", body_result);
+  output.push(format!("      has a matching body ({})", body_result));
 }
 
 fn walkdir(dir: &Path) -> anyhow::Result<Vec<anyhow::Result<Box<dyn Pact + Send + Sync>>>> {
@@ -417,9 +420,14 @@ fn walkdir(dir: &Path) -> anyhow::Result<Vec<anyhow::Result<Box<dyn Pact + Send
     Ok(pacts)
 }
 
-fn display_body_mismatch(expected: &Box<dyn Interaction>, actual: &Box<dyn Interaction>, path: &str) {
+fn display_body_mismatch(
+  expected: &Box<dyn Interaction>,
+  actual: &Box<dyn Interaction>,
+  path: &str,
+  output: &mut Vec<String>
+) {
   if expected.contents_for_verification().content_type().unwrap_or_default().is_json() {
-    println!("{}", pact_matching::json::display_diff(
+    output.push(pact_matching::json::display_diff(
       &expected.contents_for_verification().str_value().to_string(),
       &actual.contents_for_verification().str_value().to_string(),
       path, "    "));
@@ -521,36 +529,45 @@ fn filter_consumers(consumers: &[String], res: &Result<(Box<dyn Pact + Send + Sy
   consumers.is_empty() || res.is_err() || consumers.contains(&res.as_ref().unwrap().0.consumer().name)
 }
 
-/// Options to use when running the verification
+/// Options for publishing results to the Pact Broker
 #[derive(Debug, Clone)]
-pub struct VerificationOptions<F> where F: RequestFilterExecutor {
-  /// If results should be published back to the broker
-  pub publish: bool,
+pub struct PublishOptions {
   /// Provider version being published
   pub provider_version: Option<String>,
   /// Build URL to associate with the published results
   pub build_url: Option<String>,
-  /// Request filter callback
-  pub request_filter: Option<Arc<F>>,
   /// Tags to use when publishing results
   pub provider_tags: Vec<String>,
+  /// Provider branch used when publishing results
+  pub provider_branch: Option<String>,
+}
+
+impl Default for PublishOptions {
+  fn default() -> Self {
+    PublishOptions {
+      provider_version: None,
+      build_url: None,
+      provider_tags: vec![],
+      provider_branch: None,
+    }
+  }
+}
+
+/// Options to use when running the verification
+#[derive(Debug, Clone)]
+pub struct VerificationOptions<F> where F: RequestFilterExecutor {
+  /// Request filter callback
+  pub request_filter: Option<Arc<F>>,
   /// Ignore invalid/self-signed SSL certificates
   pub disable_ssl_verification: bool,
   /// Timeout in ms for verification requests and state callbacks
   pub request_timeout: u64,
-  /// Provider branch used when publishing results
-  pub provider_branch: Option<String>,
 }
 
 impl <F: RequestFilterExecutor> Default for VerificationOptions<F> {
   fn default() -> Self {
     VerificationOptions {
-      publish: false,
-      provider_version: None,
-      build_url: None,
       request_filter: None,
-      provider_tags: vec![],
-      provider_branch: None,
       disable_ssl_verification: false,
       request_timeout: 5000
     }
@@ -563,12 +580,13 @@ const VERIFICATION_NOTICE_AFTER_SUCCESSFUL_RESULT_AND_NO_PUBLISH: &str = "after_
 const VERIFICATION_NOTICE_AFTER_ERROR_RESULT_AND_PUBLISH: &str = "after_verification:success_false_published_true";
 const VERIFICATION_NOTICE_AFTER_ERROR_RESULT_AND_NO_PUBLISH: &str = "after_verification:success_false_published_false";
 
-fn display_notices(context: &Option<PactVerificationContext>, stage: &str) {
+fn process_notices(context: &Option<PactVerificationContext>, stage: &str, result: &mut VerificationExecutionResult) {
   if let Some(c) = context {
+    result.notices = c.verification_properties.notices.clone();
     for notice in &c.verification_properties.notices {
       if let Some(when) = notice.get("when") {
         if when.as_str() == stage {
-          println!("{}", notice.get("text").unwrap_or(&"".to_string()));
+          result.output.push(notice.get("text").unwrap_or(&"".to_string()).clone());
         }
       }
     }
@@ -581,13 +599,15 @@ pub fn verify_provider<F: RequestFilterExecutor, S: ProviderStateExecutor>(
   source: Vec<PactSource>,
   filter: FilterInfo,
   consumers: Vec<String>,
-  options: VerificationOptions<F>,
+  verification_options: &VerificationOptions<F>,
+  publish_options: Option<&PublishOptions>,
   provider_state_executor: &Arc<S>,
   metrics_data: Option<VerificationMetrics>
 ) -> anyhow::Result<bool> {
   match tokio::runtime::Builder::new_multi_thread().enable_all().build() {
     Ok(runtime) => runtime.block_on(
-      verify_provider_async(provider_info, source, filter, consumers, options, provider_state_executor, metrics_data)),
+      verify_provider_async(provider_info, source, filter, consumers, verification_options, publish_options, provider_state_executor, metrics_data)
+    ).map(|result| result.result),
     Err(err) => {
       error!("Verify provider process failed to start the tokio runtime: {}", err);
       Ok(false)
@@ -601,10 +621,11 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
   source: Vec<PactSource>,
   filter: FilterInfo,
   consumers: Vec<String>,
-  options: VerificationOptions<F>,
+  verification_options: &VerificationOptions<F>,
+  publish_options: Option<&PublishOptions>,
   provider_state_executor: &Arc<S>,
   metrics_data: Option<VerificationMetrics>
-) -> anyhow::Result<bool> {
+) -> anyhow::Result<VerificationExecutionResult> {
   pact_matching::matchers::configure_core_catalogue();
 
   LOG_ID.scope(format!("verify:{}", provider_info.name), async {
@@ -613,6 +634,9 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
     let mut results: Vec<(Option<String>, Result<(), MismatchResult>)> = vec![];
     let mut pending_errors: Vec<(String, MismatchResult)> = vec![];
     let mut errors: Vec<(String, MismatchResult)> = vec![];
+
+    let mut verification_result = VerificationExecutionResult::new();
+
     for pact_result in pact_results {
       match pact_result {
         Ok((pact, context, pact_source)) => {
@@ -627,30 +651,45 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
             }
           }
 
-          display_notices(&context, VERIFICATION_NOTICE_BEFORE);
+          process_notices(&context, VERIFICATION_NOTICE_BEFORE, &mut verification_result);
 
-          println!("\nVerifying a pact between {} and {}",
-          Style::new().bold().paint(pact.consumer().name.clone()),
-          Style::new().bold().paint(pact.provider().name.clone()));
+          verification_result.output.push(format!("\nVerifying a pact between {} and {}",
+            Style::new().bold().paint(pact.consumer().name.clone()),
+            Style::new().bold().paint(pact.provider().name.clone())));
 
           if pact.interactions().is_empty() {
-            println!("         {}", Yellow.paint("WARNING: Pact file has no interactions"));
+            verification_result.output.push(
+              Yellow.paint("WARNING: Pact file has no interactions").to_string()
+            );
           } else {
             let pending = match &context {
               Some(context) => context.verification_properties.pending,
               None => false
             };
-            match verify_pact_internal(&provider_info, &filter, pact, &options,
-                                       &provider_state_executor.clone(), pending).await {
-              Ok(result) => for result in &result.results {
-                results.push((result.interaction_id.clone(), result.result.clone()));
-                if let Err(error) = &result.result {
-                  if result.pending {
-                    pending_errors.push((result.description.clone(), error.clone()));
-                  } else {
-                    errors.push((result.description.clone(), error.clone()));
+            let verify_result = verify_pact_internal(
+              &provider_info,
+              &filter,
+              pact,
+              &verification_options,
+              &provider_state_executor.clone(),
+              pending
+            ).await;
+            match verify_result {
+              Ok(result) => {
+                for result in &result.results {
+                  results.push((result.interaction_id.clone(), result.result.clone()));
+                  if let Err(error) = &result.result {
+                    if result.pending {
+                      pending_errors.push((result.description.clone(), error.clone()));
+                    } else {
+                      errors.push((result.description.clone(), error.clone()));
+                    }
                   }
                 }
+
+                for output in &result.output {
+                  verification_result.output.push(output.clone());
+                }
               }
               Err(err) => {
                 if pending {
@@ -663,19 +702,19 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
               }
             }
 
-            if options.publish {
-              publish_result(&results, &pact_source, &options).await;
+            if let Some(publish) = publish_options {
+              publish_result(&results, &pact_source, &publish).await;
 
               if !errors.is_empty() || !pending_errors.is_empty() {
-                display_notices(&context, VERIFICATION_NOTICE_AFTER_ERROR_RESULT_AND_PUBLISH);
+                process_notices(&context, VERIFICATION_NOTICE_AFTER_ERROR_RESULT_AND_PUBLISH, &mut verification_result);
               } else {
-                display_notices(&context, VERIFICATION_NOTICE_AFTER_SUCCESSFUL_RESULT_AND_PUBLISH);
+                process_notices(&context, VERIFICATION_NOTICE_AFTER_SUCCESSFUL_RESULT_AND_PUBLISH, &mut verification_result);
               }
             } else {
               if !errors.is_empty() || pending_errors.is_empty() {
-                display_notices(&context, VERIFICATION_NOTICE_AFTER_ERROR_RESULT_AND_NO_PUBLISH);
+                process_notices(&context, VERIFICATION_NOTICE_AFTER_ERROR_RESULT_AND_NO_PUBLISH, &mut verification_result);
               } else {
-                display_notices(&context, VERIFICATION_NOTICE_AFTER_SUCCESSFUL_RESULT_AND_NO_PUBLISH);
+                process_notices(&context, VERIFICATION_NOTICE_AFTER_SUCCESSFUL_RESULT_AND_NO_PUBLISH, &mut verification_result);
               }
             }
           }
@@ -687,22 +726,6 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
       }
     };
 
-    if !pending_errors.is_empty() {
-      println!("\nPending Failures:\n");
-      print_errors(&pending_errors);
-      println!("\nThere were {} non-fatal pact failures on pending pacts or interactions (see docs.pact.io/pending for more information)\n", pending_errors.len());
-    }
-
-    let result = if !errors.is_empty() {
-      println!("\nFailures:\n");
-      print_errors(&errors);
-      println!("\nThere were {} pact failures\n", errors.len());
-      Ok(false)
-    } else {
-      println!();
-      Ok(true)
-    };
-
     let metrics_data = metrics_data.unwrap_or_else(|| VerificationMetrics {
       test_framework: "pact-rust".to_string(),
       app_name: "pact_verifier".to_string(),
@@ -715,24 +738,51 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
       app_version: metrics_data.app_version
     });
 
+    for (error, result) in &errors {
+      verification_result.errors.push((error.clone(), result.into()));
+    }
+    for (error, result) in &pending_errors {
+      verification_result.pending_errors.push((error.clone(), result.into()));
+    }
+
+    if !pending_errors.is_empty() {
+      verification_result.output.push("\nPending Failures:\n".to_string());
+      process_errors(&pending_errors, &mut verification_result.output);
+      verification_result.output.push(format!("\nThere were {} non-fatal pact failures on pending pacts or interactions (see docs.pact.io/pending for more information)\n", pending_errors.len()));
+    }
+
+    if !errors.is_empty() {
+      verification_result.output.push("\nFailures:\n".to_string());
+      process_errors(&errors, &mut verification_result.output);
+      verification_result.output.push(format!("\nThere were {} pact failures\n", errors.len()));
+      verification_result.result = false;
+    } else {
+      verification_result.output.push(String::default());
+      verification_result.result = true;
+    };
+
+    for line in &verification_result.output {
+      println!("{line}");
+    }
+
     shutdown_plugins();
 
-    result
+    Ok(verification_result)
   }).await
 }
 
-fn print_errors(errors: &Vec<(String, MismatchResult)>) {
+fn process_errors(errors: &Vec<(String, MismatchResult)>, output: &mut Vec<String>) {
   for (i, &(ref description, ref mismatch)) in errors.iter().enumerate() {
     match *mismatch {
-        MismatchResult::Error(ref err, _) => println!("{}) {} - {}\n", i + 1, description, err),
+        MismatchResult::Error(ref err, _) => output.push(format!("{}) {} - {}\n", i + 1, description, err)),
         MismatchResult::Mismatches { ref mismatches, ref expected, ref actual, .. } => {
-          println!("{}) {}", i + 1, description);
+          output.push(format!("{}) {}", i + 1, description));
 
           let mut j = 1;
           for (_, mut mismatches) in &mismatches.into_iter().group_by(|m| m.mismatch_type()) {
             let mismatch = mismatches.next().unwrap();
-            println!("    {}.{}) {}", i + 1, j, mismatch.summary());
-            println!("           {}", mismatch.ansi_description());
+            output.push(format!("    {}.{}) {}", i + 1, j, mismatch.summary()));
+            output.push(format!("           {}", mismatch.ansi_description()));
             for mismatch in mismatches.sorted_by(|m1, m2| {
               match (m1, m2) {
                 (Mismatch::QueryMismatch { parameter: p1, .. }, Mismatch::QueryMismatch { parameter: p2, .. }) => Ord::cmp(&p1, &p2),
@@ -742,11 +792,11 @@ fn print_errors(errors: &Vec<(String, MismatchResult)>) {
                 _ => Ord::cmp(m1, m2)
               }
             }) {
-              println!("           {}", mismatch.ansi_description());
+              output.push(format!("           {}", mismatch.ansi_description()));
             }
 
             if let Mismatch::BodyMismatch{ref path, ..} = mismatch {
-              display_body_mismatch(expected, actual, path);
+              display_body_mismatch(expected, actual, path, output);
             }
 
             j += 1;
@@ -861,7 +911,9 @@ pub struct VerificationInteractionResult {
 /// Result of verifying a Pact
 pub struct VerificationResult {
   /// Results that occurred
-  pub results: Vec<VerificationInteractionResult>
+  pub results: Vec<VerificationInteractionResult>,
+  /// Output from the verification
+  pub output: Vec<String>
 }
 
 /// Internal function, public for testing purposes
@@ -874,6 +926,7 @@ pub async fn verify_pact_internal<'a, F: RequestFilterExecutor, S: ProviderState
   pending: bool
 ) -> anyhow::Result<VerificationResult> {
   let interactions = pact.interactions();
+  let mut output = vec![];
 
   let results: Vec<(Box<dyn Interaction + Send + Sync>, Result<Option<String>, MismatchResult>)> =
     futures::stream::iter(interactions.iter().map(|i| (&pact, i)))
@@ -897,24 +950,24 @@ pub async fn verify_pact_internal<'a, F: RequestFilterExecutor, S: ProviderState
     description.push_str(" - ");
     description.push_str(&interaction.description());
 
-    println!();
+    output.push(String::default());
     if interaction.pending() {
-      println!("  {} {}", interaction.description(), Yellow.paint("[PENDING]"));
+      output.push(format!("  {} {}", interaction.description(), Yellow.paint("[PENDING]")));
     } else {
-      println!("  {}", interaction.description());
+      output.push(format!("  {}", interaction.description()));
     };
 
     if interaction.is_v4() {
       if let Some(interaction) = interaction.as_v4() {
-        display_comments(interaction)
+        process_comments(interaction, &mut output)
       }
     }
 
     if let Some(interaction) = interaction.as_request_response() {
-      display_request_response_result(&interaction, &match_result)
+      process_request_response_result(&interaction, &match_result, &mut output)
     }
     if let Some(interaction) = interaction.as_message() {
-      display_message_result(&interaction, &match_result)
+      process_message_result(&interaction, &match_result, &mut output)
     }
 
     match match_result {
@@ -937,33 +990,33 @@ pub async fn verify_pact_internal<'a, F: RequestFilterExecutor, S: ProviderState
     }
   }
 
-  println!();
+  output.push(String::default());
 
-  Ok(VerificationResult { results: errors })
+  Ok(VerificationResult { results: errors, output: output.clone() })
 }
 
-fn display_comments(interaction: Box<dyn V4Interaction>) {
+fn process_comments(interaction: Box<dyn V4Interaction>, output: &mut Vec<String>) {
   let comments = interaction.comments();
   if !comments.is_empty() {
     if let Some(testname) = comments.get("testname") {
       let s = json_to_string(testname);
       if !s.is_empty() {
-        println!("\n  Test Name: {}", s);
+        output.push(format!("\n  Test Name: {}", s));
       }
     }
     if let Some(comment_text) = comments.get("text") {
       match comment_text {
         Value::Array(comment_text) => if !comment_text.is_empty() {
-          println!("\n  Comments:");
+          output.push("\n  Comments:".to_string());
           for value in comment_text {
-            println!("    {}", json_to_string(value));
+            output.push(json_to_string(value));
           }
-          println!();
+          output.push(String::default());
         }
         Value::String(comment) => if !comment.is_empty() {
-          println!("\n  Comments:");
-          println!("    {}", comment);
-          println!();
+          output.push("\n  Comments:".to_string());
+          output.push(comment.clone());
+          output.push(String::default());
         }
         _ => {}
       }
@@ -971,10 +1024,10 @@ fn display_comments(interaction: Box<dyn V4Interaction>) {
   }
 }
 
-async fn publish_result<F: RequestFilterExecutor>(
+async fn publish_result(
   results: &[(Option<String>, Result<(), MismatchResult>)],
   source: &PactSource,
-  options: &VerificationOptions<F>
+  options: &PublishOptions
 ) {
   if let PactSource::BrokerUrl(_, broker_url, auth, links) = source.clone() {
     info!("Publishing verification results back to the Pact Broker");
